# File generated from our OpenAPI spec by Stainless.

from __future__ import annotations

from .external_plan_id_update_params import (
    ExternalPlanIDUpdateParams as ExternalPlanIDUpdateParams,
)
